@@ -14,14 +14,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+from abc import ABC, abstractmethod
 import logging
 
 log = logging.getLogger("subiquitycore.controller")
 
 
-class BaseController:
+class BaseController(ABC):
     """Base class for controllers."""
 
+    signals = []
+
     def __init__(self, common):
         self.ui = common['ui']
         self.signal = common['signal']
@@ -37,5 +40,10 @@ class BaseController:
             signals.append((sig, getattr(self, cb)))
         self.signal.connect_signals(signals)
 
+    @abstractmethod
+    def cancel(self):
+        pass
+
+    @abstractmethod
     def default(self):
-        raise NotImplementedError(self.default)
+        pass
